@@ -18,16 +18,18 @@
 # MA 02110-1301 USA.
 #
 
+import gtk
 import gtk.glade
-import gnomevfs
 
 def on_email(about, mail):
-    gnomevfs.url_show("mailto:%s" % mail)
+    if hasattr(gtk, "show_uri"):
+        gtk.show_uri(None, "mailto:%s" % mail, gtk.get_current_event_time())
 
 gtk.about_dialog_set_email_hook(on_email)
 
 def on_url(about, link):
-    gnomevfs.url_show(link)
+    if hasattr(gtk, "show_uri"):
+        gtk.show_uri(None, link, gtk.get_current_event_time())
 
 gtk.about_dialog_set_url_hook(on_url)
 
