@@ -18,6 +18,7 @@
 # MA 02110-1301 USA.
 #
 
+import logging
 import threading
 import gtk
 import gtk.gdk
@@ -36,7 +37,9 @@ class vmmAsyncJob(gobject.GObject):
         def run(self):
             threading.Thread.run(self)
 
-    def __init__(self, config, callback, args=None, text=_("Please wait a few moments..."), title=_("Operation in progress")):
+    def __init__(self, config, callback, args=None,
+                 text=_("Please wait a few moments..."),
+                 title=_("Operation in progress")):
         self.__gobject_init__()
         self.config = config
 
@@ -47,11 +50,14 @@ class vmmAsyncJob(gobject.GObject):
         self.topwin.set_title(title)
         self.topwin.hide()
 
+        # Callback sets this if there is an error
+        self._error_info = None
         self.stage = self.window.get_widget("pbar-stage")
         self.pbar = self.window.get_widget("pbar")
 
         args.append(self)
         self.bg_thread = vmmAsyncJob.asyncJobWorker(callback, args)
+        self.bg_thread.setDaemon(True)
         self.is_pulsing = True
 
     def run(self):
@@ -62,6 +68,14 @@ class vmmAsyncJob(gobject.GObject):
         gtk.main()
         gobject.source_remove(timer)
         timer = 0
+
+        if self.bg_thread.isAlive():
+            # This can happen if the user closes the whole app while the
+            # async dialog is running. This forces us to clean up properly
+            # and not leave a dead process around.
+            logging.debug("Forcing main_quit from async job.")
+            self._exit_if_necessary(force_exit=True)
+
         self.topwin.destroy()
 
     def pulse_pbar(self, progress="", stage=None):
@@ -75,7 +89,7 @@ class vmmAsyncJob(gobject.GObject):
                 self.stage.set_text(_("Processing..."))
         finally:
             gtk.gdk.threads_leave()
-            
+
 
     def set_pbar_fraction(self, frac, progress, stage=None):
         # callback for progress meter when file size is known
@@ -87,6 +101,10 @@ class vmmAsyncJob(gobject.GObject):
             else:
                 self.stage.set_text(_("Processing..."))
             self.pbar.set_text(progress)
+            if frac > 1:
+                frac = 1.0
+            if frac < 0:
+                frac = 0
             self.pbar.set_fraction(frac)
         finally:
             gtk.gdk.threads_leave()
@@ -106,15 +124,23 @@ class vmmAsyncJob(gobject.GObject):
         finally:
             gtk.gdk.threads_leave()
 
+    def set_error(self, error, details):
+        self._error_info = (error, details)
+
+    def get_error(self):
+        if not self._error_info:
+            return (None, None)
+        return self._error_info
+
     def exit_if_necessary(self):
         gtk.gdk.threads_enter()
         try:
-            return self._exit_if_necessary(self)
+            return self._exit_if_necessary()
         finally:
             gtk.gdk.threads_leave()
 
-    def _exit_if_necessary(self, unused=None):
-        if self.bg_thread.isAlive():
+    def _exit_if_necessary(self, force_exit=False):
+        if self.bg_thread.isAlive() and not force_exit:
             if(self.is_pulsing):
                 self.pbar.pulse()
             return True
