@@ -62,6 +62,9 @@ class vmmErrorDialog (gtk.MessageDialog):
             self.vbox.pack_start (expander)
             expander.show ()
 
+    def set_parent(self, parent):
+        self.set_transient_for(parent)
+
     def response_cb(self, src, ignore):
         src.hide()
 
@@ -80,21 +83,18 @@ class vmmErrorDialog (gtk.MessageDialog):
         else:
             self.run()
 
-    def val_err(self, text1, text2=None, title=None):
+    def _show_ok(self, dialog_type, text1, text2, title):
         def response_destroy(src, ignore):
             src.destroy()
 
         if self.val_err_box:
             self.val_err_box.destroy()
 
-        self.val_err_box = gtk.MessageDialog(self.parent,
+        self.val_err_box = gtk.MessageDialog(self.get_transient_for(),
                                              gtk.DIALOG_DESTROY_WITH_PARENT,
-                                             gtk.MESSAGE_ERROR,
+                                             dialog_type,
                                              gtk.BUTTONS_OK, text1)
 
-        if title is None:
-            title = _("Input Error")
-        logging.debug("Validation Error: %s" % text1)
         self.val_err_box.set_title(title)
         if text2 is not None:
             self.val_err_box.format_secondary_text(text2)
@@ -103,11 +103,21 @@ class vmmErrorDialog (gtk.MessageDialog):
         self.val_err_box.connect("response", response_destroy)
         return False
 
+    def val_err(self, text1, text2=None, title=None):
+        logging.debug("Validation Error: %s" % text1)
+        if title is None:
+            title = _("Input Error")
+        return self._show_ok(gtk.MESSAGE_ERROR, text1, text2, title)
+
+    def show_info(self, text1, text2=None, title=None):
+        if title is None:
+            title = ""
+        return self._show_ok(gtk.MESSAGE_INFO, text1, text2, title)
 
     def _show_warning(self, buttons, text1, text2):
-        message_box = gtk.MessageDialog(self.parent, \
-                                        gtk.DIALOG_DESTROY_WITH_PARENT, \
-                                        gtk.MESSAGE_WARNING, \
+        message_box = gtk.MessageDialog(self.get_transient_for(),
+                                        gtk.DIALOG_DESTROY_WITH_PARENT,
+                                        gtk.MESSAGE_WARNING,
                                         buttons, text1)
         if text2 != None:
             message_box.format_secondary_text(text2)
@@ -124,3 +134,64 @@ class vmmErrorDialog (gtk.MessageDialog):
     def ok_cancel(self, text1, text2=None):
         return self._show_warning(gtk.BUTTONS_OK_CANCEL, text1, text2)
 
+    def warn_chkbox(self, text1, text2=None, chktext=None, buttons=None):
+        chkbox = vmmCheckDialog(self.get_transient_for(),
+                                gtk.MESSAGE_WARNING, buttons)
+        return chkbox.show_chkbox(text1, text2, chktext)
+
+    def err_chkbox(self, text1, text2=None, chktext=None, buttons=None):
+        chkbox = vmmCheckDialog(self.get_transient_for(),
+                                gtk.MESSAGE_ERROR, buttons)
+        return chkbox.show_chkbox(text1, text2, chktext)
+
+class vmmCheckDialog (gtk.MessageDialog):
+    def __init__ (self, parent=None, typ=gtk.MESSAGE_INFO,
+                  buttons=None):
+        if not buttons:
+            if typ == gtk.MESSAGE_WARNING:
+                buttons = gtk.BUTTONS_OK_CANCEL
+            else:
+                buttons = gtk.BUTTONS_OK
+
+        gtk.MessageDialog.__init__ (self, parent, 0, typ, buttons)
+
+        self.connect("response", self.response_cb)
+        self.connect("delete-event", self.hide_on_delete)
+        self.set_title("")
+
+        self.chk_vbox = gtk.VBox(False, False)
+        self.chk_vbox.set_spacing(0)
+
+        self.chk_align = gtk.Alignment()
+        self.chk_align.set_padding(0, 0, 62, 0)
+        self.chk_align.add(self.chk_vbox)
+
+        self.chk_align.show_all()
+        self.vbox.pack_start(self.chk_align)
+
+    def response_cb(self, src, ignore):
+        src.hide()
+
+    def show_chkbox(self, text1, text2=None, chktext=None):
+        chkbox = None
+        res = None
+
+        self.hide()
+        for c in self.chk_vbox.get_children():
+            self.chk_vbox.remove(c)
+
+        self.set_property("text", text1)
+
+        if text2:
+            self.format_secondary_text(text2)
+
+        if chktext:
+            chkbox = gtk.CheckButton(chktext)
+            self.chk_vbox.add(chkbox)
+            chkbox.show()
+
+        res = self.run() in [ gtk.RESPONSE_YES, gtk.RESPONSE_OK ]
+        if chktext:
+            res = [res, chkbox.get_active()]
+
+        return res
